.de ZN
.ie t \fB\^\\$1\^\fR\\$2
.el \fI\^\\$1\^\fP\\$2
..
.TH XSGISTEREO 3X11 "Release 6" "X Version 11" "X FUNCTIONS"
.SH NAME
XSGIStereoQueryExtension, XSGIStereoQueryVersion, XSGIQueryStereoMode,
XSGISetStereoMode, XSGISetStereoBuffer \- X SGI stereo functions
.SH SYNTAX
.nf
.LP
#include <X11/extensions/SGIStereo.h>
.LP
Bool XSGIStereoQueryExtension (
    	Display * dpy,
    	int * first_event,
    	int * first_error);
.LP
Status XSGIStereoQueryVersion (
	Display * dpy,
    	int * major_version,
    	int * minor_version);
.LP
int XSGIQueryStereoMode(
	Display * dpy,
	Window win);
.LP
Status XSGISetStereoMode (
	Display *dpy,
	Window win,
	unsigned short height,
	unsigned short offset,
	int mode);
.LP
Status XSGISetStereoBuffer (
	Display *dpy, 
	Window win, 
	int buffer);
.LP
.fi
.SH ARGUMENTS
.IP \fIdpy\fP 1i
Specifies the connection to the X server.
.IP \fIfirst_event\fP 1i
Returns the code of the first custom event type created by the extension.
.IP \fIfirst_error\fP 1i
Returns the code of the first custom error defined by the extension.
.IP \fImajor_version\fP 1i
Returns major version number for the extension.
.IP \fIminor_version\fP 1i
Returns minor version number for the extension.
.IP \fIwin\fP 1i
Specifies the window on whose screen you want to set or inquire the stereo mode.
.IP \fIheight\fP 1i
.br
.ns
.IP \fIoffset\fP 1i
Specify the height of each stereo buffer, and the vertical offset between
each stereo buffer.
.IP \fImode\fP 1i
Specifies the stereo mode to be set. Valid values are STEREO_OFF (0) to turn off stereo,
STEREO_BOTTOM (1) to turn on stereo and use the bottom half of the screen as the primary buffer,
STEREO_TOP (2) to turn on stereo and use the top half of the screen as the primary buffer.
.IP \fIbuffer\fP 1i
Identifies which buffer the GL will render to. 

.SH DESCRIPTION
\fIXSGIStereoQueryExtension\fP is used to check whether the X server supports the Stereo extension.
If the Stereo extension is present, this routine will return True. If the extension involves
additional event types, the base event type code is returned in \fIfirst_event\fP. Otherwise, zero is
returned in \fIfirst_event\fP. If the extension involves additional error codes, the base error code
is returned in \fIfirst_error\fP. Otherwise, zero is returned.
.LP
\fIXSGIStereoQueryVersion\fP is used to get the version number of the Stereo extension supported by
the X server.
.LP
\fIXSGIQueryStereoMode\fP is used to query the server for the current stereo mode. If the server
supports the SGI Stereo extension, but \fIwin\fP's screen does not, this routine will return
X_STEREO_UNSUPPORTED (-1). If \fIwin\fP's screen supports the SGI Stereo extension, this routine will
return one of STEREO_OFF (0), STEREO_BOTTOM (1) or STEREO_TOP (2).
.LP
\fIXSGIQueryStereoMode\fP can generate \fIBadDrawable\fP.
.LP
\fIXSGISetStereoMode\fP is used to inform the X server that full-screen
stereo is being turned on or off. It does not cause the actual video timing
to change. Rather, if turning on stereo, this procedure informs the server
that all future X rendering is to be duplicated from the primary buffer to the
secondary buffer. Calling this procedure also causes the cursor to be restricted
to the primary buffer. Non-stereo GL clients will render to the left eye, with the
right eye cleared to black.
.LP
Conversely, if turning off stereo, this procedure informs the server to return to
the normal state, with X rendering allowed to the full screen and unrestricting the
cursor.
.LP
If turning on stereo, \fIheight\fP and \fIoffset\fP describe the height of each stereo
buffer and the vertical offset between buffers. Typically, these values should be 492 and 532
respectively. \fIheight\fP and \fIoffset\fP are ignored if turning off stereo.
.LP
Stereo is turned off by setting \fImode\fP to STEREO_OFF (0), and is turned on by setting \fImode\fP to
STEREO_BOTTOM or STEREO_TOP (1 or 2 respectively). Setting \fImode\fP to STEREO_BOTTOM causes the
bottom half of the screen to become the primary
buffer, setting it to STEREO_TOP causes the top half of the screen to become the primary buffer.
To switch from using the top half of the screen as the primary buffer to using the
bottom half of the screen as the primary buffer, \fIXSGISetStereoMode\fP should first be 
called with \fImode\fP set to STEREO_OFF.
.LP
NOTE: The application will not normally call this procedure. Instead, it will call
setmonitor, which will update both the X server state and the actual video format.
.LP
\fIXSGISetStereoMode\fP can generate \fIBadDrawable\fP and \fIBadRequest\fP.

\fIXSGISetStereoBuffer\fP is used to inform the X server of which eye buffer the GL
will render to next. The first time it's called for a window, it also informs the server
that the window is a stereo window.
\fIbuffer\fP identifies which buffer to set up. Valid values are STEREO_BUFFER_LEFT and
STEREO_BUFFER_RIGHT. If stereo mode is turned on (by calling XSGISetStereoMode), a call
to XSGISetStereoBuffer will cause the X server to set the clip and window origin
appropriately for the GL client. Since these will not be valid until the server has
completed its task, the application should call XSync before attempting to render
to the eye buffer. Failure to do so could cause GL rendering to the wrong eye buffer.
.LP.LP
\fIXSGISetStereoBuffer\fP can generate \fIBadDrawable\fP and \fIBadValue\fP errors.
.LP
Applications calling one of these routines should link with libXext.
.SH SEE ALSO
	setmon, setmonitor, viewport, winset

.in -.5i
.fi
.sp
